function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }

function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

import React, { useEffect, useReducer, useCallback } from "react";
import Context from "./Context";
import { doc, onSnapshot, getFirestore } from "firebase/firestore";
var LOADING_CHANGED = "LOADING_CHANGED";
var ERROR = "ERROR";
var VALUE_CHANGE = "VALUE_CHANGED";
var CLEAR = "CLEAR";
var CLEAR_ALL = "CLEAR_ALL";

function reducer(state, action) {
  var _extends2, _extends3, _extends4;

  var type = action.type,
      path = action.path,
      value = action.value,
      _action$isLoading = action.isLoading,
      isLoading = _action$isLoading === void 0 ? false : _action$isLoading,
      _action$error = action.error,
      error = _action$error === void 0 ? false : _action$error,
      _action$hasError = action.hasError,
      hasError = _action$hasError === void 0 ? false : _action$hasError;

  switch (type) {
    case LOADING_CHANGED:
      return _extends({}, state, (_extends2 = {}, _extends2[path] = _extends({}, state[path], {
        isLoading: isLoading
      }), _extends2));

    case ERROR:
      return _extends({}, state, (_extends3 = {}, _extends3[path] = _extends({}, state[path], {
        error: error,
        hasError: hasError,
        isLoading: isLoading
      }), _extends3));

    case VALUE_CHANGE:
      return _extends({}, state, (_extends4 = {}, _extends4[path] = _extends({}, state[path], {
        value: value,
        isLoading: isLoading,
        error: error,
        hasError: hasError
      }), _extends4));

    case CLEAR:
      var clearedKey = state[path],
          rest = _objectWithoutPropertiesLoose(state, [path].map(_toPropertyKey));

      return _extends({}, rest);

    case CLEAR_ALL:
      return {};

    default:
      throw new Error();
  }
}

function getInitState(persistKey) {
  var persistedValues = {};

  try {
    persistedValues = JSON.parse(localStorage.getItem(persistKey || "")) || {};
  } catch (error) {
    console.warn(error);
  }

  return persistedValues;
}

var inits = {};

var setInit = function setInit(path, unsub) {
  inits[path] = unsub;
};

var removeInit = function removeInit(path) {
  inits[path] = false;
};

var Provider = function Provider(_ref) {
  var children = _ref.children,
      _ref$persistKey = _ref.persistKey,
      persistKey = _ref$persistKey === void 0 ? "firebase_docs" : _ref$persistKey;

  var _useReducer = useReducer(reducer, getInitState(persistKey)),
      state = _useReducer[0],
      dispatch = _useReducer[1];

  useEffect(function () {
    try {
      localStorage.setItem(persistKey, JSON.stringify(state));
    } catch (error) {
      console.warn(error);
    }
  }, [state, persistKey]);
  var getRef = useCallback(function (path) {
    var db = getFirestore();

    if (typeof path === "string" || path instanceof String) {
      return doc.apply(void 0, [db].concat(path.split("/")));
    } else if (path instanceof Array) {
      return doc.apply(void 0, [db].concat(path));
    } else {
      return path;
    }
  }, []);
  var getLocation = useCallback(function (path) {
    if (typeof path === "string" || path instanceof String) {
      return path;
    } else if (path instanceof Array) {
      return path.join("/");
    } else {
      return doc(path).path;
    }
  }, []);
  var watchDoc = useCallback(function (reference, alias) {
    var ref = getRef(reference);
    var path = alias || getLocation(reference);

    if (path.length < 1) {
      return;
    }

    if (inits[path]) {
      return;
    }

    dispatch({
      type: LOADING_CHANGED,
      path: path,
      isLoading: true
    });
    var unsub = onSnapshot(ref, function (snapshot) {
      dispatch({
        type: VALUE_CHANGE,
        path: path,
        value: snapshot.data(),
        isLoading: false
      });
    }, function (error) {
      dispatch({
        type: ERROR,
        path: path,
        isLoading: false,
        error: error,
        hasError: true
      });
    });
    setInit(path, unsub);
  }, [getLocation, getRef]);
  var unwatchDoc = useCallback(function (reference) {
    var path = getLocation(reference);
    inits[path] && inits[path]();
    removeInit(path);
  }, [getLocation]);
  var getDoc = useCallback(function (path, defaultValue) {
    return state[path] ? state[path].value : defaultValue;
  }, [state]);
  var isDocLoading = useCallback(function (path) {
    return state[path] ? state[path].isLoading : false;
  }, [state]);
  var getDocError = useCallback(function (path) {
    return state[path] ? state[path].error : false;
  }, [state]);
  var hasDocError = useCallback(function (path) {
    return state[path] ? state[path].hasError : false;
  }, [state]);
  var clearDoc = useCallback(function (reference) {
    var path = getLocation(reference);
    unwatchDoc(path);
    dispatch({
      type: CLEAR,
      path: path
    });
  }, [getLocation, unwatchDoc]);
  var clearAllDocs = useCallback(function () {
    Object.keys(inits).map(function (k) {
      inits[k].unsub && inits[k].unsub();
      return k;
    });
    dispatch({
      type: CLEAR_ALL
    });
  }, []);
  return /*#__PURE__*/React.createElement(Context.Provider, {
    value: {
      watchDoc: watchDoc,
      unwatchDoc: unwatchDoc,
      getDoc: getDoc,
      clearDoc: clearDoc,
      clearAllDocs: clearAllDocs,
      isDocLoading: isDocLoading,
      hasDocError: hasDocError,
      getDocError: getDocError
    }
  }, children);
};

export default Provider;