import _regeneratorRuntime from "@babel/runtime/regenerator";

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }

function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

/* eslint-disable default-case */
import Context from "./Context";
import React, { useEffect, useReducer, useCallback } from "react";
import { getDatabase, ref, onChildAdded, onChildChanged, onChildRemoved, get, off } from "firebase/database";
var LOADING_CHANGED = "LOADING_CHANGED";
var ERROR = "ERROR";
var VALUE_CHANGE = "VALUE_CHANGED";
var CLEAR = "CLEAR";
var CLEAR_ALL = "CLEAR_ALL";
var CHILD_ADDED = "CHILD_ADDED";
var CHILD_CHANGED = "CHILD_CHANGED";
var CHILD_REMOVED = "CHILD_REMOVED";
var inits = {};

var setInit = function setInit(path) {
  inits[path] = true;
};

var removeInit = function removeInit(path) {
  inits[path] = false;
};

function list(list, action) {
  if (list === void 0) {
    list = [];
  }

  var payload = action.payload;

  switch (action.type) {
    case CHILD_ADDED:
      return list.findIndex(function (d) {
        return d.key === payload.key;
      }) === -1 ? [].concat(list, [payload]) : [].concat(list);

    case CHILD_CHANGED:
      return list.map(function (child) {
        return payload.key === child.key ? payload : child;
      });

    case CHILD_REMOVED:
      return list.filter(function (child) {
        return payload.key !== child.key;
      });
  }
}

function reducer(state, action) {
  var _extends2, _extends3, _extends4;

  var type = action.type,
      path = action.path,
      value = action.value,
      _action$isLoading = action.isLoading,
      isLoading = _action$isLoading === void 0 ? false : _action$isLoading,
      _action$error = action.error,
      error = _action$error === void 0 ? false : _action$error,
      _action$hasError = action.hasError,
      hasError = _action$hasError === void 0 ? false : _action$hasError;

  switch (type) {
    case LOADING_CHANGED:
      return _extends({}, state, (_extends2 = {}, _extends2[path] = _extends({}, state[path], {
        isLoading: isLoading
      }), _extends2));

    case ERROR:
      return _extends({}, state, (_extends3 = {}, _extends3[path] = _extends({}, state[path], {
        error: error,
        hasError: hasError,
        isLoading: isLoading
      }), _extends3));

    case VALUE_CHANGE:
      return _extends({}, state, (_extends4 = {}, _extends4[path] = _extends({}, state[path], {
        value: value,
        isLoading: isLoading,
        error: error,
        hasError: hasError
      }), _extends4));

    case CLEAR:
      var clearedKey = state[path],
          rest = _objectWithoutPropertiesLoose(state, [path].map(_toPropertyKey));

      return _extends({}, rest);

    case CHILD_ADDED:
    case CHILD_CHANGED:
    case CHILD_REMOVED:
      console.log("state[path]", state[path]);

      if (state[path]) {
        var _extends5;

        return _extends({}, state, (_extends5 = {}, _extends5[path] = _extends({}, state[path], {
          value: list(state[path].value, action)
        }), _extends5));
      } else {
        return state;
      }

    case CLEAR_ALL:
      return {};

    default:
      throw new Error();
  }
}

function getInitState(persistKey) {
  var persistedValues = {};

  try {
    persistedValues = JSON.parse(localStorage.getItem(persistKey) || "") || {};
  } catch (error) {
    console.warn(error);
  }

  return persistedValues;
}

var Provider = function Provider(_ref) {
  var children = _ref.children,
      _ref$persistKey = _ref.persistKey,
      persistKey = _ref$persistKey === void 0 ? "firebase_lists" : _ref$persistKey;

  var _useReducer = useReducer(reducer, getInitState(persistKey)),
      state = _useReducer[0],
      dispatch = _useReducer[1];

  var db = getDatabase();
  useEffect(function () {
    try {
      localStorage.setItem(persistKey, JSON.stringify(state));
    } catch (error) {
      console.warn(error);
    }
  }, [state, persistKey]);
  var getRef = useCallback(function (path) {
    if (typeof path === "string" || path instanceof String) {
      return ref(db, path);
    } else {
      return path;
    }
  }, [db]);
  var getLocation = useCallback(function (path) {
    if (typeof path === "string" || path instanceof String) {
      return path;
    } else {
      return path.toString().substring(ref(db).root.toString().length);
    }
  }, [db]);
  var watchList = useCallback( /*#__PURE__*/function () {
    var _ref2 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(reference, alias) {
      var ref, path, listenForChanges, handleError, handleChange, _list, snapshot;

      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              ref = getRef(reference);
              path = alias || getLocation(reference);

              if (!(path.length < 1)) {
                _context.next = 4;
                break;
              }

              return _context.abrupt("return");

            case 4:
              if (!inits[path]) {
                _context.next = 6;
                break;
              }

              return _context.abrupt("return");

            case 6:
              listenForChanges = false; // We can't awaid that the single child listeners get calld for every chils
              // but we can use this to not change the state after the inital call
              // because we already have all data we got trough the once call

              handleError = function handleError(error) {
                dispatch({
                  type: ERROR,
                  path: path,
                  isLoading: false,
                  error: error,
                  hasError: true
                });
                removeInit(path);
              };

              handleChange = function handleChange(s, type) {
                if (listenForChanges) {
                  dispatch({
                    type: type,
                    path: path,
                    payload: {
                      key: s.key,
                      val: s.val()
                    }
                  });
                }
              };

              setInit(path);
              dispatch({
                type: LOADING_CHANGED,
                path: path,
                isLoading: true
              });
              onChildAdded(ref, function (s) {
                return handleChange(s, CHILD_ADDED);
              }, handleError);
              onChildChanged(ref, function (s) {
                return handleChange(s, CHILD_CHANGED);
              }, handleError);
              onChildRemoved(ref, function (s) {
                return handleChange(s, CHILD_REMOVED);
              }, handleError);
              _context.prev = 14;
              _context.prev = 15;
              _list = [];
              _context.next = 19;
              return get(getRef(reference));

            case 19:
              snapshot = _context.sent;
              snapshot.forEach(function (snap) {
                _list.push({
                  key: snap.key,
                  val: snap.val()
                });
              });
              dispatch({
                type: VALUE_CHANGE,
                path: path,
                value: _list,
                isLoading: false
              });
              _context.next = 27;
              break;

            case 24:
              _context.prev = 24;
              _context.t0 = _context["catch"](15);
              console.log("Error loading inital data", _context.t0);

            case 27:
              _context.prev = 27;
              dispatch({
                type: LOADING_CHANGED,
                isLoading: false
              });
              return _context.finish(27);

            case 30:
              listenForChanges = true;
              _context.next = 36;
              break;

            case 33:
              _context.prev = 33;
              _context.t1 = _context["catch"](14);
              handleError(_context.t1);

            case 36:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, null, [[14, 33], [15, 24, 27, 30]]);
    }));

    return function (_x, _x2) {
      return _ref2.apply(this, arguments);
    };
  }(), [getLocation, getRef]);
  var unwatchList = useCallback(function (reference) {
    var ref = getRef(reference);
    var path = getLocation(reference);

    if (path.length < 1) {
      return;
    }

    off(ref);
    removeInit(path);
  }, [getRef, getLocation]);
  var getList = useCallback(function (path) {
    return state[path] && state[path].value ? state[path].value : [];
  }, [state]);
  var isListLoading = useCallback(function (path) {
    return state[path] ? state[path].isLoading : false;
  }, [state]);
  var getListError = useCallback(function (path) {
    return state[path] ? state[path].error : false;
  }, [state]);
  var hasListError = useCallback(function (path) {
    return state[path] ? state[path].hasError : false;
  }, [state]);
  var clearList = useCallback(function (reference) {
    var ref = getRef(reference);
    var path = getLocation(reference);
    unwatchList(ref);
    dispatch({
      type: CLEAR,
      path: path
    });
  }, [getRef, getLocation, unwatchList]);
  var clearAllLists = useCallback(function () {
    off(ref(db));
    dispatch({
      type: CLEAR_ALL
    });
  }, [db]);
  return /*#__PURE__*/React.createElement(Context.Provider, {
    value: {
      watchList: watchList,
      unwatchList: unwatchList,
      getList: getList,
      clearList: clearList,
      clearAllLists: clearAllLists,
      isListLoading: isListLoading,
      hasListError: hasListError,
      getListError: getListError
    }
  }, children);
};

export default Provider;