"use strict";

exports.__esModule = true;
exports["default"] = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _Context = _interopRequireDefault(require("./Context"));

var _react = _interopRequireWildcard(require("react"));

var _firestore = require("firebase/firestore");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }

function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

var LOADING_CHANGED = "LOADING_CHANGED";
var ERROR = "ERROR";
var VALUE_CHANGE = "VALUE_CHANGED";
var CLEAR = "CLEAR";
var CLEAR_ALL = "CLEAR_ALL";
var CHILD_ADDED = "CHILD_ADDED";
var CHILD_CHANGED = "CHILD_CHANGED";
var CHILD_REMOVED = "CHILD_REMOVED";

function list(list, action) {
  if (list === void 0) {
    list = [];
  }

  var payload = action.payload,
      type = action.type;

  switch (type) {
    case CHILD_ADDED:
      return list.findIndex(function (d) {
        return d.id === payload.id;
      }) === -1 ? [].concat(list, [payload]) : [].concat(list);

    case CHILD_CHANGED:
      return list.map(function (child) {
        return payload.id === child.id ? payload : child;
      });

    case CHILD_REMOVED:
      return list.filter(function (child) {
        return payload.id !== child.id;
      });
  }
}

function reducer(state, action) {
  var _extends2, _extends3, _extends4, _extends5;

  var type = action.type,
      path = action.path,
      value = action.value,
      _action$isLoading = action.isLoading,
      isLoading = _action$isLoading === void 0 ? false : _action$isLoading,
      _action$error = action.error,
      error = _action$error === void 0 ? false : _action$error,
      _action$hasError = action.hasError,
      hasError = _action$hasError === void 0 ? false : _action$hasError;

  switch (type) {
    case LOADING_CHANGED:
      return _extends({}, state, (_extends2 = {}, _extends2[path] = _extends({}, state[path], {
        isLoading: isLoading
      }), _extends2));

    case ERROR:
      return _extends({}, state, (_extends3 = {}, _extends3[path] = _extends({}, state[path], {
        error: error,
        hasError: hasError,
        isLoading: isLoading
      }), _extends3));

    case VALUE_CHANGE:
      return _extends({}, state, (_extends4 = {}, _extends4[path] = _extends({}, state[path], {
        value: value,
        isLoading: isLoading,
        error: error,
        hasError: hasError
      }), _extends4));

    case CLEAR:
      var clearedKey = state[path],
          rest = _objectWithoutPropertiesLoose(state, [path].map(_toPropertyKey));

      return _extends({}, rest);

    case CHILD_ADDED:
    case CHILD_CHANGED:
    case CHILD_REMOVED:
      return _extends({}, state, (_extends5 = {}, _extends5[path] = _extends({}, state[path], {
        value: list(state[path].value, action)
      }), _extends5));

    case CLEAR_ALL:
      return {};

    default:
      throw new Error();
  }
}

function getInitState(persistKey) {
  var persistedValues = {};

  try {
    persistedValues = JSON.parse(localStorage.getItem(persistKey || "")) || {};
  } catch (error) {
    console.warn(error);
  }

  return persistedValues;
}

var inits = {};

var setInit = function setInit(path, unsub) {
  inits[path] = unsub;
};

var removeInit = function removeInit(path) {
  inits[path] = false;
};

var getPath = function getPath(ref) {
  return ref.path;
};

var getLocation = function getLocation(path) {
  if (typeof path === "string" || path instanceof String) {
    return path;
  } else {
    return getPath(path);
  }
};

var unwatchCol = function unwatchCol(reference) {
  var path = getLocation(reference);
  inits[path] && inits[path]();
  removeInit(path);
};

var Provider = function Provider(_ref) {
  var children = _ref.children,
      _ref$persistKey = _ref.persistKey,
      persistKey = _ref$persistKey === void 0 ? "firebase_cols" : _ref$persistKey;

  var _useReducer = (0, _react.useReducer)(reducer, getInitState(persistKey)),
      state = _useReducer[0],
      dispatch = _useReducer[1];

  var db = (0, _firestore.getFirestore)();
  (0, _react.useEffect)(function () {
    try {
      localStorage.setItem(persistKey, JSON.stringify(state));
    } catch (error) {
      console.warn(error);
    }
  }, [state, persistKey]);
  var getRef = (0, _react.useCallback)(function (path) {
    if (typeof path === "string" || path instanceof String) {
      return (0, _firestore.collection)(db, path);
    } else {
      return path;
    }
  }, [db]);
  var watchCol = (0, _react.useCallback)( /*#__PURE__*/function () {
    var _ref2 = _asyncToGenerator( /*#__PURE__*/_regenerator["default"].mark(function _callee(reference, alias) {
      var ref, path, handleError, handleChange, unsub;
      return _regenerator["default"].wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              ref = getRef(reference);
              path = alias || getLocation(reference);

              if (!(path.length < 1)) {
                _context.next = 4;
                break;
              }

              return _context.abrupt("return");

            case 4:
              if (!inits[path]) {
                _context.next = 6;
                break;
              }

              return _context.abrupt("return");

            case 6:
              handleError = function handleError(error) {
                dispatch({
                  type: ERROR,
                  path: path,
                  isLoading: false,
                  error: error,
                  hasError: true
                });
                removeInit(path);
              };

              handleChange = function handleChange(doc, type) {
                dispatch({
                  type: type,
                  path: path,
                  payload: {
                    id: doc.id,
                    data: doc.data()
                  }
                });
              };

              dispatch({
                type: LOADING_CHANGED,
                path: path,
                isLoading: true
              });

              try {
                unsub = (0, _firestore.onSnapshot)(ref, function (snapshot) {
                  setInit(path, unsub);
                  dispatch({
                    type: LOADING_CHANGED,
                    path: path,
                    isLoading: false
                  });
                  snapshot.docChanges().forEach(function (change) {
                    if (change.type === "added") {
                      handleChange(change.doc, CHILD_ADDED);
                    }

                    if (change.type === "modified") {
                      handleChange(change.doc, CHILD_CHANGED);
                    }

                    if (change.type === "removed") {
                      handleChange(change.doc, CHILD_REMOVED);
                    }
                  });
                }, handleError);
              } catch (error) {
                handleError(error);
              }

            case 10:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));

    return function (_x, _x2) {
      return _ref2.apply(this, arguments);
    };
  }(), [getRef]);
  var getCol = (0, _react.useCallback)(function (path) {
    return state[path] && state[path].value ? state[path].value : [];
  }, [state]);
  var isColLoading = (0, _react.useCallback)(function (path) {
    return state[path] ? state[path].isLoading : false;
  }, [state]);
  var getColError = (0, _react.useCallback)(function (path) {
    return state[path] ? state[path].error : false;
  }, [state]);
  var hasColError = (0, _react.useCallback)(function (path) {
    return state[path] ? state[path].hasError : false;
  }, [state]);
  var clearCol = (0, _react.useCallback)(function (reference) {
    var ref = getRef(reference);
    var path = getLocation(reference);
    unwatchCol(ref);
    dispatch({
      type: CLEAR,
      path: path
    });
  }, [getRef]);
  var clearAllCols = (0, _react.useCallback)(function () {
    dispatch({
      type: CLEAR_ALL
    });
  }, []);
  return /*#__PURE__*/_react["default"].createElement(_Context["default"].Provider, {
    value: {
      watchCol: watchCol,
      unwatchCol: unwatchCol,
      getCol: getCol,
      clearCol: clearCol,
      clearAllCols: clearAllCols,
      isColLoading: isColLoading,
      hasColError: hasColError,
      getColError: getColError
    }
  }, children);
};

var _default = Provider;
exports["default"] = _default;
module.exports = exports.default;