"use strict";

exports.__esModule = true;
exports["default"] = void 0;

var _react = _interopRequireWildcard(require("react"));

var _Context = _interopRequireDefault(require("./Context"));

var _database = require("firebase/database");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }

function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function reducer(state, action) {
  var _extends2, _extends3, _extends4;

  var type = action.type,
      path = action.path,
      value = action.value,
      _action$isLoading = action.isLoading,
      isLoading = _action$isLoading === void 0 ? false : _action$isLoading,
      _action$error = action.error,
      error = _action$error === void 0 ? false : _action$error,
      _action$hasError = action.hasError,
      hasError = _action$hasError === void 0 ? false : _action$hasError;

  switch (type) {
    case "loading_changed":
      return _extends({}, state, (_extends2 = {}, _extends2[path] = _extends({}, state[path], {
        isLoading: isLoading
      }), _extends2));

    case "error_changed":
      return _extends({}, state, (_extends3 = {}, _extends3[path] = _extends({}, state[path], {
        error: error,
        hasError: hasError,
        isLoading: isLoading
      }), _extends3));

    case "value_changed":
      return _extends({}, state, (_extends4 = {}, _extends4[path] = _extends({}, state[path], {
        value: value,
        isLoading: isLoading,
        error: error,
        hasError: hasError
      }), _extends4));

    case "clear":
      var clearedKey = state[path],
          rest = _objectWithoutPropertiesLoose(state, [path].map(_toPropertyKey));

      return _extends({}, rest);

    case "clear_all":
      return {};

    default:
      throw new Error();
  }
}

function getInitState(persistKey) {
  var persistedValues = {};

  try {
    persistedValues = JSON.parse(localStorage.getItem(persistKey) || "") || {};
  } catch (error) {
    console.warn(error);
  }

  return persistedValues;
}

var Provider = function Provider(_ref) {
  var children = _ref.children,
      _ref$persistKey = _ref.persistKey,
      persistKey = _ref$persistKey === void 0 ? "firebase_paths" : _ref$persistKey;

  var _useReducer = (0, _react.useReducer)(reducer, getInitState(persistKey)),
      state = _useReducer[0],
      dispatch = _useReducer[1];

  var db = (0, _database.getDatabase)();
  (0, _react.useEffect)(function () {
    try {
      localStorage.setItem(persistKey, JSON.stringify(state));
    } catch (error) {
      console.warn(error);
    }
  }, [state, persistKey]);
  var watchPath = (0, _react.useCallback)(function (path, onChange) {
    if (path.length < 1) {
      return;
    }

    dispatch({
      type: "loading_changed",
      path: path,
      isLoading: true
    });
    (0, _database.onValue)((0, _database.ref)(db, path), function (snapshot) {
      dispatch({
        type: "value_changed",
        path: path,
        value: snapshot.val(),
        isLoading: false
      });

      if (onChange) {
        onChange(snapshot.val());
      }
    }, function (error) {
      dispatch({
        type: "error_changed",
        path: path,
        isLoading: false,
        error: error,
        hasError: true
      });
    });
  }, [db]);
  var unwatchPath = (0, _react.useCallback)(function (path) {
    if (path.length < 1) {
      return;
    }

    (0, _database.off)((0, _database.ref)(db, path));
  }, [db]);
  var getPath = (0, _react.useCallback)(function (path, defaultValue) {
    return state[path] ? state[path].value : defaultValue;
  }, [state]);
  var isPathLoading = (0, _react.useCallback)(function (path) {
    return state[path] ? state[path].isLoading : false;
  }, [state]);
  var getPathError = (0, _react.useCallback)(function (path) {
    return state[path] ? state[path].error : false;
  }, [state]);
  var hasPathError = (0, _react.useCallback)(function (path) {
    return state[path] ? state[path].hasError : false;
  }, [state]);
  var clearPath = (0, _react.useCallback)(function (path) {
    unwatchPath(path);
    dispatch({
      type: "clear",
      path: path
    });
  }, [unwatchPath]);
  var clearAllPaths = (0, _react.useCallback)(function () {
    (0, _database.off)((0, _database.ref)(db));
    dispatch({
      type: "clear_all"
    });
  }, [db]);
  return /*#__PURE__*/_react["default"].createElement(_Context["default"].Provider, {
    value: {
      watchPath: watchPath,
      unwatchPath: unwatchPath,
      getPath: getPath,
      clearPath: clearPath,
      clearAllPaths: clearAllPaths,
      isPathLoading: isPathLoading,
      hasPathError: hasPathError,
      getPathError: getPathError
    }
  }, children);
};

var _default = Provider;
exports["default"] = _default;
module.exports = exports.default;