"use strict";

exports.__esModule = true;
exports.isGranted = exports.isAnyGranted = exports.defaultUserData = void 0;

// get the default user data
var defaultUserData = function defaultUserData(user) {
  if (user != null) {
    return {
      displayName: user.displayName,
      email: user.email,
      photoURL: user.photoURL,
      emailVerified: user.emailVerified,
      isAnonymous: user.isAnonymous,
      uid: user.uid,
      providerData: user.providerData,
      isAuthenticated: true
    };
  } else {
    return {
      isAuthenticated: false
    };
  }
};

exports.defaultUserData = defaultUserData;

var isGranted = function isGranted(auth, grant) {
  var _ref = auth || {},
      _ref$grants = _ref.grants,
      grants = _ref$grants === void 0 ? [] : _ref$grants,
      _ref$isAdmin = _ref.isAdmin,
      isAdmin = _ref$isAdmin === void 0 ? false : _ref$isAdmin;

  if (isAdmin) {
    return true;
  }

  if (!grants) {
    return false;
  }

  return !!grants[grant];
};

exports.isGranted = isGranted;

var isAnyGranted = function isAnyGranted(auth, grantsToCheck) {
  if (grantsToCheck === void 0) {
    grantsToCheck = [];
  }

  var _ref2 = auth || {},
      _ref2$grants = _ref2.grants,
      grants = _ref2$grants === void 0 ? [] : _ref2$grants,
      _ref2$isAdmin = _ref2.isAdmin,
      isAdmin = _ref2$isAdmin === void 0 ? false : _ref2$isAdmin;

  if (isAdmin) {
    return true;
  }

  if (!grants) {
    return false;
  }

  if (!grantsToCheck) {
    return false;
  }

  var granted = false;

  for (var i = 0; i < grantsToCheck.length; i++) {
    var grant = grantsToCheck[i];

    if (isGranted(auth, grant)) {
      granted = true;
    }
  }

  return granted;
};

exports.isAnyGranted = isAnyGranted;